Subject: bugfix - fix broken output in expanded aligned format, when data are too short

From: Pavel Stehule <[email-masked]>
Date: Mon, 23 Mar 2026 20:42:35 +0100

Hi

There is a bug in fe_utils

(2026-03-23 20:36:21) postgres=# \pset
border                   2
columns                  0
csv_fieldsep             ','
display_false            'f'
display_true             't'
expanded                 on
fieldsep                 '|'
fieldsep_zero            off
footer                   on
format                   aligned
linestyle                unicode
null                     '∅'
numericlocale            off
pager                    1
pager_min_lines          0
recordsep                '\n'
recordsep_zero           off
tableattr
title
tuples_only              off
unicode_border_linestyle single
unicode_column_linestyle single
unicode_header_linestyle double
xheader_width            full

(2026-03-23 20:36:56) postgres=# select * from foo;
kuku
┌────┬────┐
│ a  │ b  │
╞════╪════╡
│ 10 │ 20 │
│ 10 │ 20 │
└────┴────┘
(2 rows)

(2026-03-23 20:36:58) postgres=# \x
Expanded display is on.
(2026-03-23 20:37:01) postgres=# select * from foo;
┌─[ RECORD 1 ]─┐
│ a │ 10 │
│ b │ 20 │
╞═[ RECORD 2 ]═╡
│ a │ 10 │
│ b │ 20 │
└───┴────┘

the bugfix is really short

pavel@nemesis:~/src/postgresql$ git diff master
diff --git a/src/fe_utils/print.c b/src/fe_utils/print.c
index 12d969e8666..58a04a902d5 100644
--- a/src/fe_utils/print.c
+++ b/src/fe_utils/print.c
@@ -1445,7 +1445,7 @@ print_aligned_vertical(const printTableContent *cont,
    /*
     * Calculate available width for data in wrapped mode
     */
-   if (cont->opt->format == PRINT_WRAPPED)
+   if (cont->opt->format == PRINT_WRAPPED || cont->opt->format ==
PRINT_ALIGNED)
    {
        unsigned int swidth,
                    rwidth = 0,

after fix:

(2026-03-23 20:40:11) postgres=# \x
Expanded display is on.
(2026-03-23 20:40:13) postgres=# select * from foo;
┌─[ RECORD 1 ]─┐
│ a │ 10       │
│ b │ 20       │
╞═[ RECORD 2 ]═╡
│ a │ 10       │
│ b │ 20       │
└───┴──────────┘

Regards

Pavel
From 131261620f35997be1cf2b4726577c2c284335d2 Mon Sep 17 00:00:00 2001
From: "[email-masked]" <[email-masked]>
Date: Mon, 23 Mar 2026 20:39:31 +0100
Subject: [PATCH] fix expended aligned format

---
 src/fe_utils/print.c | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/src/fe_utils/print.c b/src/fe_utils/print.c
index 12d969e8666..58a04a902d5 100644
--- a/src/fe_utils/print.c
+++ b/src/fe_utils/print.c
@@ -1445,7 +1445,7 @@ print_aligned_vertical(const printTableContent *cont,
 	/*
 	 * Calculate available width for data in wrapped mode
 	 */
-	if (cont->opt->format == PRINT_WRAPPED)
+	if (cont->opt->format == PRINT_WRAPPED || cont->opt->format == PRINT_ALIGNED)
 	{
 		unsigned int swidth,
 					rwidth = 0,
-- 
2.53.0

---

From: Pavel Stehule <[email-masked]>
Date: Tue, 24 Mar 2026 06:46:34 +0100

Hi

new version

* fixed unwanted forcing to wrapped mode
* regress test

Regards

Pavel
From 8894b8720b0c89f97b3f439c5ae0bc957d550e17 Mon Sep 17 00:00:00 2001
From: "[email-masked]" <[email-masked]>
Date: Tue, 24 Mar 2026 06:42:18 +0100
Subject: [PATCH] The output of thin table is broken in expanded mode (when
 header is wider than row) and when aligned mode is used. The wrapped mode is
 ok. This patch fixes this issue.

---
 src/fe_utils/print.c               |  4 ++--
 src/test/regress/expected/psql.out | 26 ++++++++++++++++++++++++++
 src/test/regress/sql/psql.sql      | 11 +++++++++++
 3 files changed, 39 insertions(+), 2 deletions(-)

diff --git a/src/fe_utils/print.c b/src/fe_utils/print.c
index 12d969e8666..7d7eb7dd041 100644
--- a/src/fe_utils/print.c
+++ b/src/fe_utils/print.c
@@ -1445,7 +1445,7 @@ print_aligned_vertical(const printTableContent *cont,
 	/*
 	 * Calculate available width for data in wrapped mode
 	 */
-	if (cont->opt->format == PRINT_WRAPPED)
+	if (cont->opt->format == PRINT_WRAPPED || cont->opt->format == PRINT_ALIGNED)
 	{
 		unsigned int swidth,
 					rwidth = 0,
@@ -1517,7 +1517,7 @@ print_aligned_vertical(const printTableContent *cont,
 			if (width < rwidth)
 				width = rwidth;
 
-			if (output_columns > 0)
+			if (cont->opt->format == PRINT_WRAPPED && output_columns > 0)
 			{
 				unsigned int min_width;
 
diff --git a/src/test/regress/expected/psql.out b/src/test/regress/expected/psql.out
index c8f3932edf0..dc44219631e 100644
--- a/src/test/regress/expected/psql.out
+++ b/src/test/regress/expected/psql.out
@@ -2854,6 +2854,32 @@ execute q;
 +------------------+-------------------+
 
 deallocate q;
+-- the output in expanded mode is shorter than header
+\pset border 2
+\pset expanded on
+create table psql_short_tab(a int, b int);
+insert into psql_short_tab values(10,20),(30,40);
+\pset format aligned
+select * from psql_short_tab;
++-[ RECORD 1 ]-+
+| a | 10       |
+| b | 20       |
++-[ RECORD 2 ]-+
+| a | 30       |
+| b | 40       |
++---+----------+
+
+\pset format wrapped
+select * from psql_short_tab;
++-[ RECORD 1 ]-+
+| a | 10       |
+| b | 20       |
++-[ RECORD 2 ]-+
+| a | 30       |
+| b | 40       |
++---+----------+
+
+drop table psql_short_tab;
 \pset linestyle ascii
 \pset border 1
 -- support table for output-format tests (useful to create a footer)
diff --git a/src/test/regress/sql/psql.sql b/src/test/regress/sql/psql.sql
index dcdbd4fc020..b03a61f8656 100644
--- a/src/test/regress/sql/psql.sql
+++ b/src/test/regress/sql/psql.sql
@@ -499,6 +499,17 @@ execute q;
 
 deallocate q;
 
+-- the output in expanded mode is shorter than header
+\pset border 2
+\pset expanded on
+create table psql_short_tab(a int, b int);
+insert into psql_short_tab values(10,20),(30,40);
+\pset format aligned
+select * from psql_short_tab;
+\pset format wrapped
+select * from psql_short_tab;
+drop table psql_short_tab;
+
 \pset linestyle ascii
 \pset border 1
 
-- 
2.53.0

---

From: Chao Li <[email-masked]>
Date: Tue, 24 Mar 2026 14:12:18 +0800

> On Mar 24, 2026, at 13:46, Pavel Stehule <[email-masked]> wrote:
> 
> Hi
> 
> new version
> 
> <v20260324-1-0001-The-output-of-thin-table-is-broken-in-expanded-mode-.patch>

Thanks for the fix. The patch overall looks good to me. A couple of small comments:

1
```
 	/*
```

Since the condition has been updated, I think the comment above should be adjusted as well.

2
```
```

After this test, does it make sense to turn expanded mode off explicitly, in case it affects following tests? Today the next test happens to reset it, but maybe tomorrow a new test gets added before that point.

Best regards,
--
Chao Li (Evan)
HighGo Software Co., Ltd.
https://www.highgo.com/

---

From: Pavel Stehule <[email-masked]>
Date: Tue, 24 Mar 2026 07:31:11 +0100

> Since the condition has been updated, I think the comment above should be

done

pavel@nemesis:~/src/postgresql$ git diff
diff --git a/src/fe_utils/print.c b/src/fe_utils/print.c
index 7d7eb7dd041..dbfe437bc4c 100644
--- a/src/fe_utils/print.c
+++ b/src/fe_utils/print.c
@@ -1443,7 +1443,8 @@ print_aligned_vertical(const printTableContent *cont,
    }

    /*
-    * Calculate available width for data in wrapped mode
+    * Calculate available width for data in wrapped mode or minimal width
+    * in aligned mode
     */
    if (cont->opt->format == PRINT_WRAPPED || cont->opt->format ==
PRINT_ALIGNED)
    {

> After this test, does it make sense to turn expanded mode off explicitly,
> in case it affects following tests? Today the next test happens to reset
> it, but maybe tomorrow a new test gets added before that point.
>

There is no default pset setting in psql.sql. Following test has its own
setting. Generally this is the responsibility to every test to set the
environment how it is necessary. Maybe I am wrong, but I see, so only
database objects and prepared queries are cleaned there. There is not
problem to write \pset border 1 \pset expanded off, but following tests
starts with \pset expanded off and this can be little bit messy and
redundant

Thank you for check

Regards

Pavel

> Best regards,
From b46c5393e674f11515fe7d025effa48dc4aaf4d8 Mon Sep 17 00:00:00 2001

---
 src/fe_utils/print.c               |  7 ++++---
 src/test/regress/expected/psql.out | 26 ++++++++++++++++++++++++++
 src/test/regress/sql/psql.sql      | 11 +++++++++++
 3 files changed, 41 insertions(+), 3 deletions(-)

diff --git a/src/fe_utils/print.c b/src/fe_utils/print.c
index 12d969e8666..dbfe437bc4c 100644
--- a/src/fe_utils/print.c
+++ b/src/fe_utils/print.c
@@ -1443,9 +1443,10 @@ print_aligned_vertical(const printTableContent *cont,
 	}
 
 	/*
 	 * Calculate available width for data in wrapped mode
 	 */
-	if (cont->opt->format == PRINT_WRAPPED)
+	if (cont->opt->format == PRINT_WRAPPED || cont->opt->format == PRINT_ALIGNED)
 	{
 		unsigned int swidth,
 					rwidth = 0,
@@ -1517,7 +1518,7 @@ print_aligned_vertical(const printTableContent *cont,
 			if (width < rwidth)
 				width = rwidth;
@@ -1443,9 +1443,10 @@ print_aligned_vertical(const printTableContent *cont,
 
 	/*
-	 * Calculate available width for data in wrapped mode
+	 * Calculate available width for data in wrapped mode or minimal width
+	 * in aligned mode
 	 */

 
 deallocate q;
 
 deallocate q;

-- 
2.53.0

---

From: Pavel Stehule <[email-masked]>
Date: Tue, 24 Mar 2026 08:23:01 +0100

út 24. 3. 2026 v 8:17 odesílatel getiancheng <[email-masked]>
napsal:

>
>
> ---- Replied Message ----
> From Chao Li<[email-masked]> <[email-masked]>
> Date 3/24/2026 15:01
> To Pavel Stehule<[email-masked]> <[email-masked]>
> Cc PostgreSQL Hackers<[email-masked]>
> <[email-masked]>
> Subject Re: bugfix - fix broken output in expanded aligned format, when
> data are too short
>
>
>
> On Mar 24, 2026, at 14:31, Pavel Stehule <[email-masked]> wrote:
>
>
>
>      if (cont->opt->format == PRINT_WRAPPED || cont->opt->format == PRINT_ALIGNED)
>      {
>
>
>  2
>  ```
>  There is no default pset setting in psql.sql. Following test has its own setting. Generally this is the responsibility to every test to set the environment how it is necessary. Maybe I am wrong, but I see, so only database objects and prepared queries are cleaned there. There is not problem to write \pset border 1 \pset expanded off, but following tests starts with \pset expanded off and this can be little bit messy and redundant
>
>  Thank you for check
>
>  Regards
>
>  Pavel
>
>  Best regards,
>  --
>  Chao Li (Evan)
>  <v20260324-2-0001-The-output-of-thin-table-is-broken-in-expanded-mode-.patch>
>
>
> LGTM.
>
> Best regards,
> --
> Chao Li (Evan)
> Hi,
> Overall looks good to me. Only nitpick is that, in the commit message, "thin table" sounds a little unusual, maybe "narrow table".
>
>
changed in attached version

Thank you for check

Regards

Pavel

> Tiancheng Ge
From ec2048128dfd41a1a53e73121dc678ebeae344a8 Mon Sep 17 00:00:00 2001
From: "[email-masked]" <[email-masked]>
Date: Tue, 24 Mar 2026 06:42:18 +0100
Subject: [PATCH] The output of narrow table is broken in expanded mode (when
 header is wider than row) and when aligned mode is used. The wrapped mode is
 ok. This patch fixes this issue.

---

 	/*

 
 deallocate q;
 
 deallocate q;

-- 
2.53.0

---

From: Pavel Stehule <[email-masked]>
Date: Tue, 26 May 2026 08:22:17 +0200

Hi

fresh rebase

Regards

Pavel
From d99d82117f294329947240be4023943370220ca0 Mon Sep 17 00:00:00 2001

---

index f2dd52003c1..a5e64cb18fb 100644
 
 	/*

 
 deallocate q;
 
 deallocate q;

-- 
2.54.0

---

From: Pavel Stehule <[email-masked]>
Date: Sun, 7 Jun 2026 10:39:21 +0200

Hi

only rebase

Regards

Pavel
From 38b7d7d4eeed977e66b9f1ee7e8a3150f6bf6f3e Mon Sep 17 00:00:00 2001

---

 	/*

 
 deallocate q;
 
 deallocate q;

-- 
2.54.0

---

From: Michael Paquier <[email-masked]>
Date: Mon, 8 Jun 2026 10:36:29 +0900

> only rebase

The patch has been lying around for a couple of months, and as far as
I can see it looks incorrect in the way this calculates the column
width.  Short of some adjustments for the description of the test and
the comment you have changed, I'll double-check the whole.

Thanks for the report and the patch.
--
Michael

---

From: Michael Paquier <[email-masked]>
Date: Mon, 8 Jun 2026 14:45:40 +0900

> Thanks for the report and the patch.

Done as of 3d0d6741d810 & friends.
--
Michael

---

From: Pavel Stehule <[email-masked]>
Date: Mon, 8 Jun 2026 08:39:06 +0200

> Done as of 3d0d6741d810 & friends.

Thank you

Pavel